@@ -6,29 +6,33 @@ dPdTtr <- function(ispecies) {
   # calculate dP/dT for a phase transition
   # (argument is index of the lower-T phase)
   thermo <- get("thermo")
-  t1 <- subcrt(ispecies, P=0, T=thermo$obigt$z.T[ispecies], convert=FALSE, check.Ttr=FALSE)
-  t2 <- subcrt(ispecies+1, P=0, T=thermo$obigt$z.T[ispecies], convert=FALSE, check.Ttr=FALSE)
+  pars <- info(c(ispecies, ispecies+1), check.it=FALSE)
+  # the special handling for quartz and coesite interferece with this function,
+  # so we convert to uppercase names to prevent cgl() from calling quartz_coesite()
+  pars$name <- toupper(pars$name)
+  props <- cgl(c("G", "S", "V"), pars, P=0, T=thermo$obigt$z.T[ispecies])
   # if these aren't the same mineral all we can say is zero
   # actually, should be infinity ... the volume change is zero
-  if(as.character(t1$species$name) != as.character(t2$species$name)) return(Inf)
+  if(as.character(pars$name[1]) != as.character(pars$name[2])) return(Inf)
   # we really hope the G's are the same ...
-  #if(abs(t2$out[[1]]$G - t2$out[[1]]$G) > 0.1) warning('dP.dT: inconsistent values of G for different phases of ',ispecies,call.=FALSE)
-  dP.dT <- convert( ( t2$out[[1]]$S - t1$out[[1]]$S ) / ( t2$out[[1]]$V - t1$out[[1]]$V ), 'cm3bar' )
+  #if(abs(props[[2]]$G - props[[1]]$G) > 0.1) warning('dP.dT: inconsistent values of G for different phases of ',ispecies,call.=FALSE)
+  dP.dT <- convert( ( props[[2]]$S - props[[1]]$S ) / ( props[[2]]$V - props[[1]]$V ), 'cm3bar' )
   return(dP.dT)
 }
 
 Ttr <- function(ispecies,P=1,dPdT=NULL) {
-  # calculate a phase transition temperature
-  # taking account of P (from dP/dT)
-  T <- get("thermo")$obigt$z.T[ispecies]
+  # calculate a phase transition temperature for given P
+  TtrPr <- get("thermo")$obigt$z.T[ispecies]
+  # the constant slope, dP/dT
   if(is.null(dPdT)) dPdT <- dPdTtr(ispecies)
-  return(T + P / dPdT)
+  Pr <- 1
+  TtrPr + (P - Pr) / dPdT
 }
 
 GHS_Tr <- function(ispecies, Htr) {
   # calculate G, H, and S at Tr for cr2, cr3, ... phases 20170301
   # Htr: enthalpy(ies) of transition
-  # ispecies: the species index for cr1
+  # ispecies: the species index for cr (the lowest-T phase)
   thisinfo <- info(ispecies)
   name <- thisinfo$name
   # start from Tr (T=298.15 K)
@@ -40,26 +44,26 @@ GHS_Tr <- function(ispecies, Htr) {
   # where to store the calculated GHS at Tr
   Gf_Tr <- Hf_Tr <- S_Tr <- numeric()
   for(i in 1:(length(Htr)+1)) {
-    # check that we have the correct one of cr1, cr2, ...
-    thiscr <- paste0("cr", i)
+    # check that we have the correct one of cr, cr2, cr3, ...
+    if(i==1) thiscr <- "cr" else thiscr <- paste0("cr", i)
     if(thisinfo$state!=thiscr | thisinfo$name!=name) stop(paste("species", thisis, "is not", name, thiscr))
-    # if we're above cr1, calculate the equivalent GHS at Tr
+    # if we're above cr (lowest-T), calculate the equivalent GHS at Tr
     if(i > 1) {
       # set the starting GHS to 0 (in case they're NA - we only need the increments over temperature)
       thisinfo$G <- thisinfo$H <- thisinfo$S <- 0
       # the HS increments from 298.15 to Ttr
-      HSinc <- cgl(c("H", "S"), T=c(298.15, Ttr), ghs=thisinfo, eos=thisinfo)
+      HSinc <- cgl(c("H", "S"), parameters=thisinfo, T=c(298.15, Ttr))
       Hf_Tr <- c(Hf_Tr, Hf - diff(HSinc[[1]]$H))
       S_Tr <- c(S_Tr, S - diff(HSinc[[1]]$S))
       # plug in the calculated S_Tr to calculate the G increment correctly
       thisinfo$S <- tail(S_Tr, 1)
-      Ginc <- cgl("G", T=c(298.15, Ttr), ghs=thisinfo, eos=thisinfo)
+      Ginc <- cgl("G", parameters=thisinfo, T=c(298.15, Ttr))
       Gf_Tr <- c(Gf_Tr, Gf - diff(Ginc[[1]]$G))
     }
     # the temperature of the next transition
     Ttr <- thisinfo$T
     # the GHS increments from Tprev to Ttr
-    GHCinc <- cgl(c("G", "H", "S"), T=c(Tprev, Ttr), ghs=thisinfo, eos=thisinfo)
+    GHCinc <- cgl(c("G", "H", "S"), parameters=thisinfo, T=c(Tprev, Ttr))
     # the GHS + transition at Tr
     Gf <- Gf + diff(GHCinc[[1]]$G)
     Hf <- Hf + diff(GHCinc[[1]]$H) + Htr[i]
